# Generated by Django 2.2.3 on 2019-11-07 11:50

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [("grapple", "0002_delete_pagepreview")]

    operations = [
        migrations.CreateModel(
            name="StubModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
            options={"managed": False},
        )
    ]



import inspect

import graphene
import graphene_django

from django.utils.translation import gettext_lazy as _
from graphene.types import Int
from taggit.managers import _TaggableManager
from wagtail.search.index import class_is_indexed


class PositiveInt(Int):
    """
    GraphQL type for an integer that must be equal or greater than zero.
    """

    @staticmethod
    def parse_literal(ast, _variables=None):
        return_value = Int.parse_literal(ast, _variables=_variables)
        if return_value is not None and return_value >= 0:
            return return_value


class QuerySetList(graphene.List):
    """
    List type with arguments used by Django's query sets.

    This list setts the following arguments on itself:

    * ``id``
    * ``limit``
    * ``offset``
    * ``search_query``
    * ``order``

    :param enable_limit: Enable limit argument.
    :type enable_limit: bool
    :param enable_offset: Enable offset argument.
    :type enable_offset: bool
    :param enable_search: Enable search query argument.
    :type enable_search: bool
    :param enable_order: Enable ordering via query argument.
    :type enable_order: bool
    """

    def __init__(self, of_type, *args, **kwargs):
        enable_limit = kwargs.pop("enable_limit", True)
        enable_offset = kwargs.pop("enable_offset", True)
        enable_search = kwargs.pop("enable_search", True)
        enable_order = kwargs.pop("enable_order", True)

        # Check if the type is a Django model type. Do not perform the
        # check if value is lazy.
        if inspect.isclass(of_type) and not issubclass(
            of_type, graphene_django.DjangoObjectType
        ):
            raise TypeError(
                f"{of_type} is not a subclass of DjangoObjectType and it "
                "cannot be used with QuerySetList."
            )
        # Enable limiting on the queryset.
        if enable_limit is True and "limit" not in kwargs:
            kwargs["limit"] = graphene.Argument(
                PositiveInt, description=_("Limit a number of resulting objects.")
            )

        # Enable offset on the queryset.
        if enable_offset is True and "offset" not in kwargs:
            kwargs["offset"] = graphene.Argument(
                PositiveInt,
                description=_(
                    "Number of records skipped from the beginning of the "
                    "results set."
                ),
            )

        # Enable ordering of the queryset
        if enable_order is True and "order" not in kwargs:
            kwargs["order"] = graphene.Argument(
                graphene.String, description=_("Use the Django queryset order format.")
            )

        # If type is provided as a lazy value (e.g. using lambda), then
        # the search has to be enabled explicitly.
        if (enable_search is True and not inspect.isclass(of_type)) or (
            enable_search is True
            and inspect.isclass(of_type)
            and class_is_indexed(of_type._meta.model)
            and "search_query" not in kwargs
        ):
            kwargs["search_query"] = graphene.Argument(
                graphene.String,
                description=_("Filter the results using Wagtail's search."),
            )

        if "id" not in kwargs:
            kwargs["id"] = graphene.Argument(graphene.ID, description=_("Filter by ID"))

        super().__init__(of_type, *args, **kwargs)


class TagList(graphene.JSONString):
    """
    A tag list from the TaggableManager.
    """

    @staticmethod
    def serialize(value):
        if isinstance(value, _TaggableManager):
            return list(value.values_list("name", flat=True))
        raise ValueError("Cannot convert tags object")


class PaginationType(graphene.ObjectType):
    """
    GraphQL type for Paginated QuerySet pagination field.
    """

    total = PositiveInt(required=True)
    count = PositiveInt(required=True)
    per_page = PositiveInt(required=True)
    current_page = PositiveInt(required=True)
    prev_page = PositiveInt()
    next_page = PositiveInt()
    total_pages = PositiveInt(required=True)


class BasePaginatedType(graphene.ObjectType):
    """
    GraphQL type for Paginated QuerySet result.
    """

    items = graphene.List(graphene.String)
    pagination = graphene.Field(PaginationType)


def PaginatedQuerySet(of_type, type_class, **kwargs):
    """
    Paginated QuerySet type with arguments used by Django's query sets.

    This type setts the following arguments on itself:

    * ``id``
    * ``page``
    * ``per_page``
    * ``search_query``
    * ``order``

    :param enable_search: Enable search query argument.
    :type enable_search: bool
    :param enable_order: Enable ordering via query argument.
    :type enable_order: bool
    """

    enable_search = kwargs.pop("enable_search", True)
    enable_order = kwargs.pop("enable_order", True)
    required = kwargs.get("required", False)
    type_name = type_class if isinstance(type_class, str) else type_class.__name__
    type_name = type_name.lstrip("Stub")

    # Check if the type is a Django model type. Do not perform the
    # check if value is lazy.
    if inspect.isclass(of_type) and not issubclass(
        of_type, graphene_django.DjangoObjectType
    ):
        raise TypeError(
            f"{of_type} is not a subclass of DjangoObjectType and it "
            "cannot be used with QuerySetList."
        )

    # Enable page for Django Paginator.
    if "page" not in kwargs:
        kwargs["page"] = graphene.Argument(
            PositiveInt,
            description=_("Page of resulting objects to return."),
        )

    # Enable per_page for Django Paginator.
    if "per_page" not in kwargs:
        kwargs["per_page"] = graphene.Argument(
            PositiveInt,
            description=_("The maximum number of items to include on a page."),
        )

    # Enable ordering of the queryset
    if enable_order is True and "order" not in kwargs:
        kwargs["order"] = graphene.Argument(
            graphene.String, description=_("Use the Django QuerySet order_by format.")
        )

    # If type is provided as a lazy value (e.g. using lambda), then
    # the search has to be enabled explicitly.
    if (enable_search is True and not inspect.isclass(of_type)) or (
        enable_search is True
        and inspect.isclass(of_type)
        and class_is_indexed(of_type._meta.model)
        and "search_query" not in kwargs
    ):
        kwargs["search_query"] = graphene.Argument(
            graphene.String, description=_("Filter the results using Wagtail's search.")
        )

    if "id" not in kwargs:
        kwargs["id"] = graphene.Argument(graphene.ID, description=_("Filter by ID"))

    class PaginatedType(BasePaginatedType):
        items = graphene.List(of_type, required=required)
        pagination = graphene.Field(PaginationType, required=required)

        class Meta:
            name = type_name + "PaginatedType"

    return graphene.Field(PaginatedType, **kwargs)



from typing import Optional

import graphene

from django.contrib.contenttypes.models import ContentType
from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _
from graphene_django.types import DjangoObjectType
from wagtail.models import Page as WagtailPage
from wagtail.models import Site

from ..utils import resolve_queryset, resolve_site_by_hostname, resolve_site_by_id
from .pages import get_page_interface, get_specific_page
from .structures import QuerySetList


class SiteObjectType(DjangoObjectType):
    pages = QuerySetList(
        graphene.NonNull(get_page_interface),
        content_type=graphene.Argument(
            graphene.String,
            description=_("Filter by content type. Uses the `app.Model` notation."),
        ),
        enable_search=True,
        required=True,
    )
    page = graphene.Field(
        get_page_interface(),
        id=graphene.ID(),
        slug=graphene.String(),
        url_path=graphene.String(),
        token=graphene.String(),
        content_type=graphene.String(),
    )

    def resolve_pages(self, info, **kwargs):
        pages = WagtailPage.objects.in_site(self).live().public().specific()

        content_type = kwargs.pop("content_type", None)
        if content_type:
            app_label, model = content_type.strip().lower().split(".")
            try:
                ctype = ContentType.objects.get(app_label=app_label, model=model)
            except ContentType.DoesNotExist:
                return (
                    WagtailPage.objects.none()
                )  # something not quite right here, bail out early
            else:
                pages = pages.filter(content_type=ctype)

        return resolve_queryset(pages, info, **kwargs)

    def resolve_page(self, info, **kwargs):
        return get_specific_page(
            id=kwargs.get("id"),
            slug=kwargs.get("slug"),
            url_path=kwargs.get("url_path"),
            token=kwargs.get("token"),
            content_type=kwargs.get("content_type"),
            site=self,
        )

    class Meta:
        model = Site


def SitesQuery():
    class Mixin:
        site = graphene.Field(
            SiteObjectType, hostname=graphene.String(), id=graphene.ID()
        )
        sites = QuerySetList(
            graphene.NonNull(SiteObjectType), enable_search=True, required=True
        )

        def resolve_sites(self, info, **kwargs) -> QuerySet[Site]:
            """
            Return all `Site` objects.
            """

            return resolve_queryset(Site.objects.all(), info, **kwargs)

        def resolve_site(self, info, **kwargs) -> Optional[Site]:
            """
            Attempt to return a single `Site` object identified by ID or
            hostname.
            """

            if id := kwargs.get("id"):
                return resolve_site_by_id(id=id)
            elif hostname := kwargs.get("hostname"):
                return resolve_site_by_hostname(
                    hostname=hostname,
                    filter_name="hostname",
                )
            return None

    return Mixin



import graphene

from graphene_django.types import DjangoObjectType
from wagtail.documents import get_document_model
from wagtail.documents.models import Document as WagtailDocument

from ..registry import registry
from ..utils import get_media_item_url, resolve_queryset
from .collections import CollectionObjectType
from .structures import QuerySetList
from .tags import TagObjectType


class DocumentObjectType(DjangoObjectType):
    """
    Base document type used if one isn't generated for the current model.
    All other node types extend this.
    """

    id = graphene.ID(required=True)
    title = graphene.String(required=True)
    file = graphene.String(required=True)
    created_at = graphene.DateTime(required=True)
    file_size = graphene.Int()
    file_hash = graphene.String(required=True)
    url = graphene.String(required=True)
    collection = graphene.Field(lambda: CollectionObjectType, required=True)
    tags = graphene.List(graphene.NonNull(lambda: TagObjectType), required=True)

    def resolve_url(self, info, **kwargs):
        """
        Get document file url.
        """
        return get_media_item_url(self)

    def resolve_tags(self, info, **kwargs):
        return self.tags.all()

    class Meta:
        model = WagtailDocument


def get_document_type():
    mdl = get_document_model()
    return registry.documents.get(mdl, DocumentObjectType)


def DocumentsQuery():
    mdl = get_document_model()
    mdl_type = get_document_type()

    class Mixin:
        document = graphene.Field(mdl_type, id=graphene.ID())
        documents = QuerySetList(
            graphene.NonNull(mdl_type),
            enable_search=True,
            required=True,
            collection=graphene.Argument(
                graphene.ID, description="Filter by collection id"
            ),
        )
        document_type = graphene.String(required=True)

        def resolve_document(self, info, id, **kwargs):
            """Returns a document given the id, if in a public collection"""
            try:
                return mdl.objects.filter(
                    collection__view_restrictions__isnull=True
                ).get(pk=id)
            except mdl.DoesNotExist:
                return None

        def resolve_documents(self, info, **kwargs):
            """Returns all documents in a public collection"""
            qs = mdl.objects.filter(collection__view_restrictions__isnull=True)
            return resolve_queryset(qs, info, **kwargs)

        def resolve_document_type(self, info, **kwargs):
            return get_document_type()

    return Mixin



from __future__ import annotations

from typing import TYPE_CHECKING

import graphene

from graphene_django import DjangoObjectType
from wagtail import VERSION as WAGTAIL_VERSION
from wagtail.images import get_image_model
from wagtail.images.models import Image as WagtailImage
from wagtail.images.models import Rendition as WagtailImageRendition

from ..registry import registry
from ..settings import grapple_settings
from ..utils import get_media_item_url, resolve_queryset
from .collections import CollectionObjectType
from .structures import QuerySetList
from .tags import TagObjectType


if TYPE_CHECKING:
    from graphql import GraphQLResolveInfo

if WAGTAIL_VERSION > (5, 0):
    from wagtail.images.utils import to_svg_safe_spec


def get_image_type():
    return registry.images.get(get_image_model(), ImageObjectType)


def get_rendition_type():
    rendition_mdl = get_image_model().renditions.rel.related_model
    return registry.images.get(rendition_mdl, ImageRenditionObjectType)


def get_rendition_field_kwargs() -> dict[str, graphene.Scalar]:
    """
    Returns a list of kwargs for the rendition field.
    Extracted for convenience, to accommodate for the conditional logic needed for various Wagtail versions.
    """
    kwargs = {
        "max": graphene.String(),
        "min": graphene.String(),
        "width": graphene.Int(),
        "height": graphene.Int(),
        "fill": graphene.String(),
        "format": graphene.String(),
        "bgcolor": graphene.String(),
        "jpegquality": graphene.Int(),
        "webpquality": graphene.Int(),
    }
    if WAGTAIL_VERSION > (5, 0):
        kwargs["preserve_svg"] = graphene.Boolean(
            description="Prevents raster image operations (e.g. `format-webp`, `bgcolor`, etc.) being applied to SVGs. "
            "More info: https://docs.wagtail.org/en/stable/topics/images.html#svg-images"
        )

    return kwargs


def get_src_set_field_kwargs() -> dict[str, graphene.Scalar]:
    """
    Returns a list of kwargs for the srcSet field.
    Extracted for convenience, to accommodate for the conditional logic needed for various Wagtail versions.
    """
    kwargs = {
        "sizes": graphene.List(graphene.Int),
        "format": graphene.String(),
    }
    if WAGTAIL_VERSION > (5, 0):
        kwargs["preserve_svg"] = graphene.Boolean(
            description="Prevents raster image operations (e.g. `format-webp`, `bgcolor`, etc.) being applied to SVGs. "
            "More info: https://docs.wagtail.org/en/stable/topics/images.html#svg-images"
        )

    return kwargs


def rendition_allowed(filter_specs: str) -> bool:
    """Checks a given rendition filter is allowed"""
    allowed_filters = grapple_settings.ALLOWED_IMAGE_FILTERS
    if allowed_filters is None or not isinstance(allowed_filters, (list, tuple)):
        return True

    return filter_specs in allowed_filters


class ImageRenditionObjectType(DjangoObjectType):
    id = graphene.ID(required=True)
    file = graphene.String(required=True)
    image = graphene.Field(lambda: get_image_type(), required=True)
    filter_spec = graphene.String(required=True)
    width = graphene.Int(required=True)
    height = graphene.Int(required=True)
    focal_point_key = graphene.String(required=True)
    focal_point = graphene.String()
    url = graphene.String(required=True)
    alt = graphene.String(required=True)
    background_position_style = graphene.String(required=True)

    class Meta:
        model = WagtailImageRendition

    def resolve_url(
        instance: WagtailImageRendition, info: GraphQLResolveInfo, **kwargs
    ):
        return instance.full_url


class ImageObjectType(DjangoObjectType):
    id = graphene.ID(required=True)
    title = graphene.String(required=True)
    file = graphene.String(required=True)
    width = graphene.Int(required=True)
    height = graphene.Int(required=True)
    created_at = graphene.DateTime(required=True)
    focal_point_x = graphene.Int()
    focal_point_y = graphene.Int()
    focal_point_width = graphene.Int()
    focal_point_height = graphene.Int()
    file_size = graphene.Int()
    file_hash = graphene.String(required=True)
    src = graphene.String(required=True, deprecation_reason="Use the `url` attribute")
    url = graphene.String(required=True)
    aspect_ratio = graphene.Float(required=True)
    sizes = graphene.String(required=True)
    collection = graphene.Field(lambda: CollectionObjectType, required=True)
    tags = graphene.List(graphene.NonNull(lambda: TagObjectType), required=True)
    rendition = graphene.Field(get_rendition_type, **get_rendition_field_kwargs())
    src_set = graphene.String(**get_src_set_field_kwargs())
    if WAGTAIL_VERSION > (5, 0):
        is_svg = graphene.Boolean(required=True)

    class Meta:
        model = WagtailImage

    def resolve_rendition(
        instance: WagtailImage, info: GraphQLResolveInfo, **kwargs
    ) -> WagtailImageRendition | None:
        """
        Render a custom rendition of the current image.
        """
        preserve_svg = kwargs.pop("preserve_svg", True)
        filter_specs = "|".join([f"{key}-{val}" for key, val in kwargs.items()])

        # Only allow the defined filters (thus renditions)
        if not rendition_allowed(filter_specs):
            raise TypeError(
                "Invalid filter specs. Check the `ALLOWED_IMAGE_FILTERS` setting."
            )

        if WAGTAIL_VERSION > (5, 0) and instance.is_svg() and preserve_svg:
            # when dealing with SVGs, we want to limit the filter specs to those that are safe
            filter_specs = to_svg_safe_spec(filter_specs)

            if not filter_specs:
                raise TypeError(
                    "No valid filter specs for SVG. "
                    "See https://docs.wagtail.org/en/stable/topics/images.html#svg-images for details."
                )

        # previously we wrapped this in a try/except SourceImageIOError block.
        # Removed to allow the error to bubble up in the response ("errors") and be handled by the user.
        return instance.get_rendition(filter_specs)

    def resolve_url(instance: WagtailImage, info: GraphQLResolveInfo, **kwargs) -> str:
        """
        Get the uploaded image url.
        """
        return get_media_item_url(instance)

    def resolve_src(self: WagtailImage, info, **kwargs) -> str:
        """
        Deprecated. Use the `url` attribute.
        """
        return get_media_item_url(self)

    def resolve_aspect_ratio(
        instance: WagtailImage, info: GraphQLResolveInfo, **kwargs
    ):
        """
        Calculate aspect ratio for the image.
        """
        return instance.width / instance.height

    def resolve_sizes(
        instance: WagtailImage, info: GraphQLResolveInfo, **kwargs
    ) -> str:
        return f"(max-width: {instance.width}px) 100vw, {instance.width}px"

    def resolve_tags(instance: WagtailImage, info: GraphQLResolveInfo, **kwargs):
        return instance.tags.all()

    def resolve_src_set(
        instance: WagtailImage,
        info: GraphQLResolveInfo,
        sizes: list[int],
        format: str | None = None,
        preserve_svg: bool = True,
        **kwargs,
    ) -> str:
        """
        Generate src set of renditions.
        """
        filter_suffix = f"|format-{format}" if format else ""
        format_kwarg = {"format": format} if format else {}
        if instance.file.name is not None:
            rendition_list = [
                ImageObjectType.resolve_rendition(
                    instance,
                    info,
                    width=width,
                    preserve_svg=preserve_svg,
                    **format_kwarg,
                )
                for width in sizes
                if rendition_allowed(f"width-{width}{filter_suffix}")
            ]

            return ", ".join(
                [
                    f"{get_media_item_url(img)} {img.width}w"
                    for img in rendition_list
                    if img is not None
                ]
            )

        return ""

    if WAGTAIL_VERSION > (5, 0):

        def resolve_is_svg(
            instance: WagtailImage, info: GraphQLResolveInfo, **kwargs
        ) -> bool:
            return instance.is_svg()


def ImagesQuery():
    mdl = get_image_model()
    mdl_type = get_image_type()

    class Mixin:
        image = graphene.Field(mdl_type, id=graphene.ID())
        images = QuerySetList(
            graphene.NonNull(mdl_type),
            enable_search=True,
            required=True,
            collection=graphene.Argument(
                graphene.ID, description="Filter by collection id"
            ),
        )
        image_type = graphene.String(required=True)

        def resolve_image(parent, info, id, **kwargs):
            """Returns an image given the id, if in a public collection"""
            try:
                return (
                    mdl.objects.filter(collection__view_restrictions__isnull=True)
                    .prefetch_renditions()
                    .get(pk=id)
                )
            except mdl.DoesNotExist:
                return None

        def resolve_images(parent, info, **kwargs):
            """Returns all images in a public collection"""
            return resolve_queryset(
                mdl.objects.filter(
                    collection__view_restrictions__isnull=True
                ).prefetch_renditions(),
                info,
                **kwargs,
            )

        # Give name of the image type, used to generate mixins
        def resolve_image_type(parent, info, **kwargs):
            return mdl_type

    return Mixin



import graphene

from graphql.validation import NoUnusedFragmentsRule
from wagtail import hooks

from .registry import registry
from .settings import grapple_settings


# HACK: Remove NoUnusedFragments validator
# Due to the way previews work on the frontend, we need to pass all
# fragments into the query even if they're not used.
# This would usually cause a validation error. There doesn't appear
# to be a nice way to disable this validator so we monkey-patch it instead.


# We can't simply override specified_rules because it's a tuple and immutable. Instead, we are
# monkey patching the NoUnusedFragmentRule.leave_document, so it doesn't do any validation.
NoUnusedFragmentsRule.leave_document = lambda self, *_args: None


def create_schema():
    """
    Root schema object that graphene is pointed at.
    It inherits its queries from each of the specific type mixins.
    """

    query_mixins = []
    for fn in hooks.get_hooks("register_schema_query"):
        fn(query_mixins)

    # ensure graphene.ObjectType is always present
    if graphene.ObjectType not in query_mixins:
        query_mixins.append(graphene.ObjectType)

    class Query(*query_mixins):
        pass

    mutation_mixins = []
    for fn in hooks.get_hooks("register_schema_mutation"):
        fn(mutation_mixins)

    if len(mutation_mixins) > 0:

        class Mutation(*mutation_mixins):
            pass

    else:
        Mutation = None

    subscription_mixins = []
    for fn in hooks.get_hooks("register_schema_subscription"):
        fn(subscription_mixins)

    if len(subscription_mixins) > 0:
        # ensure graphene.ObjectType is always present
        if graphene.ObjectType not in subscription_mixins:
            subscription_mixins.append(graphene.ObjectType)

        class Subscription(*subscription_mixins):
            pass

    else:
        Subscription = None

    return graphene.Schema(
        query=Query,
        mutation=Mutation,
        subscription=Subscription,
        types=list(registry.models.values()),
        auto_camelcase=grapple_settings.AUTO_CAMELCASE,
    )


schema = create_schema()



import graphene

from graphene_django.converter import convert_django_field
from modelcluster.contrib.taggit import ClusterTaggableManager
from taggit.managers import TaggableManager
from taggit.models import Tag

from grapple.utils import resolve_queryset

from .structures import QuerySetList


@convert_django_field.register(TaggableManager)
@convert_django_field.register(ClusterTaggableManager)
def convert_tag_manager_to_string(field, registry=None):
    return TagObjectType()


class TagObjectType(graphene.ObjectType):
    tag_id = graphene.ID(name="id", required=True)
    name = graphene.String(required=True)

    def resolve_tag_id(self, info, **kwargs):
        return self.id

    def resolve_name(self, info, **kwargs):
        return self.name


def TagsQuery():
    class Mixin:
        tag = graphene.Field(TagObjectType, id=graphene.ID())
        tags = QuerySetList(
            graphene.NonNull(TagObjectType), required=True, enable_search=False
        )

        def resolve_tag(self, info, id, **kwargs):
            try:
                return Tag.objects.get(pk=id)
            except Tag.DoesNotExist:
                return None

        def resolve_tags(self, info, **kwargs):
            return resolve_queryset(Tag.objects.all(), info, **kwargs)

    return Mixin



import graphene

from ..registry import registry


class SnippetTypes:
    # SnippetObjectType class can only be created if
    # registry.snippets.types is non-empty, and should only be created
    # once (graphene complains if we register multiple type classes
    # with identical names)
    _SnippetObjectType = None

    @classmethod
    def get_object_type(cls):
        if cls._SnippetObjectType is None and registry.snippets:

            class SnippetObjectType(graphene.Union):
                class Meta:
                    types = registry.snippets.types

            cls._SnippetObjectType = SnippetObjectType
        return cls._SnippetObjectType


def SnippetsQuery():
    SnippetObjectType = SnippetTypes.get_object_type()

    if SnippetObjectType is not None:

        class Mixin:
            snippets = graphene.List(graphene.NonNull(SnippetObjectType), required=True)
            # Return all snippets.

            def resolve_snippets(self, info, **kwargs):
                snippet_objects = []
                for snippet in registry.snippets:
                    for object in snippet._meta.model.objects.all():
                        snippet_objects.append(object)

                return snippet_objects

        return Mixin

    else:

        class Mixin:
            pass

        return Mixin



from typing import Union

from graphene.types import String
from wagtail.rich_text import RichText as WagtailRichText

from ..settings import grapple_settings


class RichText(String):
    @staticmethod
    def coerce_rich_text(rich_text: Union[str, WagtailRichText]):
        # When serializing a model instance, we get a str. When serializing a
        # RichTextBlock instance, its an instance of wagtail.rich_text.RichText already.
        if grapple_settings.RICHTEXT_FORMAT == "html":
            if isinstance(rich_text, str):
                return WagtailRichText(rich_text).__html__()
            return rich_text.__html__()
        elif isinstance(rich_text, str):
            return rich_text
        else:
            return rich_text.source

    serialize = coerce_rich_text
    parse_value = coerce_rich_text
